@@ -13,7 +13,7 @@ use std::ops::{Add, Sub, Mul, AddAssign, SubAssign, MulAssign};
 
 use itertools::{enumerate, zip};
 
-use {Ix, Ixs, Ix0, Ix1, Ix2, Ix3, IxDyn, Dim, Si};
+use {Ix, Ixs, Ix0, Ix1, Ix2, Ix3, IxDyn, Dim, Si, IxDynImpl};
 use IntoDimension;
 use RemoveAxis;
 use {ArrayView1, ArrayViewMut1};
@@ -32,8 +32,8 @@ use super::axes_of;
 /// This trait defines a number of methods and operations that can be used on
 /// dimensions and indices.
 ///
-/// ***Note:*** *Don't implement this trait.*
-pub unsafe trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
+/// **Note:** *This trait can not be implemented outside the crate*
+pub trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
     IndexMut<usize, Output=usize> +
     Add<Self, Output=Self> +
     AddAssign + for<'x> AddAssign<&'x Self> +
@@ -47,13 +47,13 @@ pub unsafe trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
     /// dimension.
     ///
     /// For the fixed size dimensions it is a fixed size array of the correct
-    /// size, which you pass by reference. For the `Vec` dimension it is
+    /// size, which you pass by reference. For the dynamic dimension it is
     /// a slice.
     ///
     /// - For `Ix1`: `[Si; 1]`
     /// - For `Ix2`: `[Si; 2]`
     /// - and so on..
-    /// - For `Vec<Ix>`: `[Si]`
+    /// - For `IxDyn`: `[Si]`
     ///
     /// The easiest way to create a `&SliceArg` is using the macro
     /// [`s![]`](macro.s!.html).
@@ -63,11 +63,10 @@ pub unsafe trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
     /// - For `Ix1`: `usize`,
     /// - For `Ix2`: `(usize, usize)`
     /// - and so on..
-    /// - For `Vec<Ix>`: `Vec<usize>`,
+    /// - For `IxDyn`: `IxDyn`
     type Pattern: IntoDimension<Dim=Self>;
-    // Next smaller dimension (if it exists)
-    #[doc(hidden)]
-    type TrySmaller: Dimension;
+    /// Next smaller dimension (if applicable)
+    type Smaller: Dimension;
     #[doc(hidden)]
     fn ndim(&self) -> usize;
 
@@ -147,6 +146,12 @@ pub unsafe trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
         strides
     }
 
+    #[doc(hidden)]
+    // Return an index of same dimensionality
+    fn zero_index(&self) -> Self {
+        Self::default()
+    }
+
     #[doc(hidden)]
     #[inline]
     fn first_index(&self) -> Option<Self> {
@@ -347,7 +352,9 @@ pub unsafe trait Dimension : Clone + Eq + Debug + Send + Sync + Default +
     }
 
     #[doc(hidden)]
-    fn try_remove_axis(&self, axis: Axis) -> Self::TrySmaller;
+    fn try_remove_axis(&self, axis: Axis) -> Self::Smaller;
+
+    private_decl!{}
 }
 
 // utility functions
@@ -365,10 +372,10 @@ fn abs_index(len: Ixs, index: Ixs) -> Ix {
 // Dimension impls
 
 
-unsafe impl Dimension for Dim<[Ix; 0]> {
+impl Dimension for Dim<[Ix; 0]> {
     type SliceArg = [Si; 0];
     type Pattern = ();
-    type TrySmaller = Self;
+    type Smaller = Self;
     // empty product is 1 -> size is 1
     #[inline]
     fn ndim(&self) -> usize { 0 }
@@ -385,16 +392,18 @@ unsafe impl Dimension for Dim<[Ix; 0]> {
         None
     }
     #[inline]
-    fn try_remove_axis(&self, _ignore: Axis) -> Self::TrySmaller {
+    fn try_remove_axis(&self, _ignore: Axis) -> Self::Smaller {
         *self
     }
+
+    private_impl!{}
 }
 
 
-unsafe impl Dimension for Dim<[Ix; 1]> {
+impl Dimension for Dim<[Ix; 1]> {
     type SliceArg = [Si; 1];
     type Pattern = Ix;
-    type TrySmaller = <Self as RemoveAxis>::Smaller;
+    type Smaller = Ix0;
     #[inline]
     fn ndim(&self) -> usize { 1 }
     #[inline]
@@ -470,15 +479,16 @@ unsafe impl Dimension for Dim<[Ix; 1]> {
         }
     }
     #[inline]
-    fn try_remove_axis(&self, axis: Axis) -> Self::TrySmaller {
+    fn try_remove_axis(&self, axis: Axis) -> Self::Smaller {
         self.remove_axis(axis)
     }
+    private_impl!{}
 }
 
-unsafe impl Dimension for Dim<[Ix; 2]> {
+impl Dimension for Dim<[Ix; 2]> {
     type SliceArg = [Si; 2];
     type Pattern = (Ix, Ix);
-    type TrySmaller = <Self as RemoveAxis>::Smaller;
+    type Smaller = Ix1;
     #[inline]
     fn ndim(&self) -> usize { 2 }
     #[inline]
@@ -596,15 +606,16 @@ unsafe impl Dimension for Dim<[Ix; 2]> {
         }
     }
     #[inline]
-    fn try_remove_axis(&self, axis: Axis) -> Self::TrySmaller {
+    fn try_remove_axis(&self, axis: Axis) -> Self::Smaller {
         self.remove_axis(axis)
     }
+    private_impl!{}
 }
 
-unsafe impl Dimension for Dim<[Ix; 3]> {
+impl Dimension for Dim<[Ix; 3]> {
     type SliceArg = [Si; 3];
     type Pattern = (Ix, Ix, Ix);
-    type TrySmaller = <Self as RemoveAxis>::Smaller;
+    type Smaller = Ix2;
     #[inline]
     fn ndim(&self) -> usize { 3 }
     #[inline]
@@ -701,17 +712,18 @@ unsafe impl Dimension for Dim<[Ix; 3]> {
         order
     }
     #[inline]
-    fn try_remove_axis(&self, axis: Axis) -> Self::TrySmaller {
+    fn try_remove_axis(&self, axis: Axis) -> Self::Smaller {
         self.remove_axis(axis)
     }
+    private_impl!{}
 }
 
 macro_rules! large_dim {
     ($n:expr, $name:ident, $($ix:ident),+) => (
-        unsafe impl Dimension for Dim<[Ix; $n]> {
+        impl Dimension for Dim<[Ix; $n]> {
             type SliceArg = [Si; $n];
             type Pattern = ($($ix,)*);
-            type TrySmaller = <Self as RemoveAxis>::Smaller;
+            type Smaller = Dim<[Ix; $n - 1]>;
             #[inline]
             fn ndim(&self) -> usize { $n }
             #[inline]
@@ -723,9 +735,10 @@ macro_rules! large_dim {
             #[inline]
             fn slice_mut(&mut self) -> &mut [Ix] { self.ixm() }
             #[inline]
-            fn try_remove_axis(&self, axis: Axis) -> Self::TrySmaller {
+            fn try_remove_axis(&self, axis: Axis) -> Self::Smaller {
                 self.remove_axis(axis)
             }
+            private_impl!{}
         }
     )
 }
@@ -734,13 +747,13 @@ large_dim!(4, Ix4, Ix, Ix, Ix, Ix);
 large_dim!(5, Ix5, Ix, Ix, Ix, Ix, Ix);
 large_dim!(6, Ix6, Ix, Ix, Ix, Ix, Ix, Ix);
 
-/// Vec<Ix> is a "dynamic" index, pretty hard to use when indexing,
+/// IxDyn is a "dynamic" index, pretty hard to use when indexing,
 /// and memory wasteful, but it allows an arbitrary and dynamic number of axes.
-unsafe impl Dimension for IxDyn
+impl Dimension for IxDyn
 {
     type SliceArg = [Si];
     type Pattern = Self;
-    type TrySmaller = <Self as RemoveAxis>::Smaller;
+    type Smaller = Self;
     #[inline]
     fn ndim(&self) -> usize { self.ix().len() }
     #[inline]
@@ -751,29 +764,39 @@ unsafe impl Dimension for IxDyn
     fn into_pattern(self) -> Self::Pattern {
         self
     }
+
+    fn zero_index(&self) -> Self {
+        const ZEROS: &'static [usize] = &[0; 4];
+        let n = self.ndim();
+        if n <= ZEROS.len() {
+            Dim(&ZEROS[..n])
+        } else {
+            Dim(vec![0; n])
+        }
+    }
+
     #[inline]
-    fn try_remove_axis(&self, axis: Axis) -> Self::TrySmaller {
+    fn try_remove_axis(&self, axis: Axis) -> Self::Smaller {
         if self.ndim() > 0 {
             self.remove_axis(axis)
         } else {
             self.clone()
         }
     }
+    private_impl!{}
 }
 
-impl<J> Index<J> for Dim<Vec<usize>>
-    where Vec<usize>: Index<J>,
+impl Index<usize> for Dim<IxDynImpl>
 {
-    type Output = <Vec<usize> as Index<J>>::Output;
-    fn index(&self, index: J) -> &Self::Output {
+    type Output = <IxDynImpl as Index<usize>>::Output;
+    fn index(&self, index: usize) -> &Self::Output {
         &self.ix()[index]
     }
 }
 
-impl<J> IndexMut<J> for Dim<Vec<usize>>
-    where Vec<usize>: IndexMut<J>,
+impl IndexMut<usize> for Dim<IxDynImpl>
 {
-    fn index_mut(&mut self, index: J) -> &mut Self::Output {
+    fn index_mut(&mut self, index: usize) -> &mut Self::Output {
         &mut self.ixm()[index]
     }
 }
