@@ -23,29 +23,34 @@ use super::ZipExt;
 use dimension::IntoDimension;
 use dimension::{axes_of, Axes, merge_axes, stride_offset};
 use iterators::{
-    new_inners,
-    new_inners_mut,
-    whole_chunks_of,
-    whole_chunks_mut_of,
+    new_lanes,
+    new_lanes_mut,
+    exact_chunks_of,
+    exact_chunks_mut_of,
+    windows
 };
 use zip::Zip;
 
 use {
     NdIndex,
+};
+use iter::{
     AxisChunksIter,
     AxisChunksIterMut,
     Iter,
     IterMut,
     IndexedIter,
     IndexedIterMut,
-    InnerIter,
-    InnerIterMut,
+    Lanes,
+    LanesMut,
     AxisIter,
     AxisIterMut,
-    WholeChunks,
-    WholeChunksMut,
+    ExactChunks,
+    ExactChunksMut,
+    Windows
 };
 use stacking::stack;
+use PrivateNew;
 
 /// # Methods For All Array Types
 impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
@@ -180,8 +185,10 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
     /// is where the rightmost index is varying the fastest.
     ///
     /// Iterator element type is `(D::Pattern, &A)`.
+    ///
+    /// See also [`Zip::indexed`](struct.Zip.html)
     pub fn indexed_iter(&self) -> IndexedIter<A, D> {
-        IndexedIter(self.view().into_elements_base())
+        IndexedIter::new(self.view().into_elements_base())
     }
 
     /// Return an iterator of indexes and mutable references to the elements of the array.
@@ -193,7 +200,7 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
     pub fn indexed_iter_mut(&mut self) -> IndexedIterMut<A, D>
         where S: DataMut,
     {
-        IndexedIterMut(self.view_mut().into_elements_base())
+        IndexedIterMut::new(self.view_mut().into_elements_base())
     }
 
 
@@ -360,8 +367,7 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
     ///     a.subview(Axis(1), 1) == ArrayView::from(&[2., 4., 6.])
     /// );
     /// ```
-    pub fn subview(&self, axis: Axis, index: Ix)
-        -> ArrayView<A, <D as RemoveAxis>::Smaller>
+    pub fn subview(&self, axis: Axis, index: Ix) -> ArrayView<A, D::Smaller>
         where D: RemoveAxis,
     {
         self.view().into_subview(axis, index)
@@ -459,38 +465,133 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
         }
     }
 
-    /// Return an iterator that traverses over all dimensions but the innermost,
-    /// and yields each inner row.
+    /// Return a producer and iterable that traverses over the *generalized*
+    /// rows of the array. For a 2D array these are the regular rows.
     ///
-    /// For example, in a 2 × 2 × 3 array, the iterator element
-    /// is a row of 3 elements (and there are 2 × 2 = 4 rows in total).
+    /// This is equivalent to `.lanes(Axis(n - 1))` where *n* is `self.ndim()`.
+    ///
+    /// For an array of dimensions *a* × *b* × *c* × ... × *l* × *m*
+    /// it has *a* × *b* × *c* × ... × *l* rows each of length *m*.
+    ///
+    /// For example, in a 2 × 2 × 3 array, each row is 3 elements long
+    /// and there are 2 × 2 = 4 rows in total.
     ///
     /// Iterator element is `ArrayView1<A>` (1D array view).
     ///
     /// ```
-    /// use ndarray::arr3;
+    /// use ndarray::{arr3, Axis, arr1};
+    ///
     /// let a = arr3(&[[[ 0,  1,  2],    // -- row 0, 0
     ///                 [ 3,  4,  5]],   // -- row 0, 1
     ///                [[ 6,  7,  8],    // -- row 1, 0
     ///                 [ 9, 10, 11]]]); // -- row 1, 1
-    /// // `inner_iter` yields the four inner rows of the 3D array.
-    /// let mut row_sums = a.inner_iter().map(|v| v.scalar_sum());
-    /// assert_eq!(row_sums.collect::<Vec<_>>(), vec![3, 12, 21, 30]);
+    ///
+    /// // `genrows` will yield the four generalized rows of the array.
+    /// for row in a.genrows() {
+    ///     /* loop body */
+    /// }
     /// ```
-    pub fn inner_iter(&self) -> InnerIter<A, D> {
-        iterators::new_inner_iter(self.view())
+    pub fn genrows(&self) -> Lanes<A, D::Smaller> {
+        let mut n = self.ndim();
+        if n == 0 { n += 1; }
+        new_lanes(self.view(), Axis(n - 1))
     }
 
-    /// Return an iterator that traverses over all dimensions but the innermost,
-    /// and yields each inner row.
+    /// Return a producer and iterable that traverses over the *generalized*
+    /// rows of the array and yields mutable array views.
     ///
-    /// Iterator element is `ArrayViewMut1<A>` (1D read-write array view).
-    pub fn inner_iter_mut(&mut self) -> InnerIterMut<A, D>
+    /// Iterator element is `ArrayView1<A>` (1D read-write array view).
+    pub fn genrows_mut(&mut self) -> LanesMut<A, D::Smaller>
         where S: DataMut
     {
-        iterators::new_inner_iter_mut(self.view_mut())
+        let mut n = self.ndim();
+        if n == 0 { n += 1; }
+        new_lanes_mut(self.view_mut(), Axis(n - 1))
     }
 
+    /// Return a producer and iterable that traverses over the *generalized*
+    /// columns of the array. For a 2D array these are the regular columns.
+    ///
+    /// This is equivalent to `.lanes(Axis(0))`.
+    ///
+    /// For an array of dimensions *a* × *b* × *c* × ... × *l* × *m*
+    /// it has *b* × *c* × ... × *l* × *m* columns each of length *a*.
+    ///
+    /// For example, in a 2 × 2 × 3 array, each column is 2 elements long
+    /// and there are 2 × 3 = 6 columns in total.
+    ///
+    /// Iterator element is `ArrayView1<A>` (1D array view).
+    ///
+    /// ```
+    /// use ndarray::{arr3, Axis, arr1};
+    ///
+    /// // The generalized columns of a 3D array:
+    /// // are directed along the 0th axis: 0 and 6, 1 and 7 and so on...
+    /// let a = arr3(&[[[ 0,  1,  2], [ 3,  4,  5]],
+    ///                [[ 6,  7,  8], [ 9, 10, 11]]]);
+    ///
+    /// // Here `gencolumns` will yield the six generalized columns of the array.
+    /// for row in a.gencolumns() {
+    ///     /* loop body */
+    /// }
+    /// ```
+    pub fn gencolumns(&self) -> Lanes<A, D::Smaller> {
+        new_lanes(self.view(), Axis(0))
+    }
+
+    /// Return a producer and iterable that traverses over the *generalized*
+    /// columns of the array and yields mutable array views.
+    ///
+    /// Iterator element is `ArrayView1<A>` (1D read-write array view).
+    pub fn gencolumns_mut(&mut self) -> LanesMut<A, D::Smaller>
+        where S: DataMut
+    {
+        new_lanes_mut(self.view_mut(), Axis(0))
+    }
+
+    /// Return a producer and iterable that traverses over all 1D lanes
+    /// pointing in the direction of `axis`.
+    ///
+    /// When the pointing in the direction of the first axis, they are *columns*,
+    /// in the direction of the last axis *rows*; in general they are all
+    /// *lanes* and are one dimensional.
+    ///
+    /// Iterator element is `ArrayView1<A>` (1D array view).
+    ///
+    /// ```
+    /// use ndarray::{arr3, aview1, Axis};
+    ///
+    /// let a = arr3(&[[[ 0,  1,  2],
+    ///                 [ 3,  4,  5]],
+    ///                [[ 6,  7,  8],
+    ///                 [ 9, 10, 11]]]);
+    ///
+    /// let inner0 = a.lanes(Axis(0));
+    /// let inner1 = a.lanes(Axis(1));
+    /// let inner2 = a.lanes(Axis(2));
+    ///
+    /// // The first lane for axis 0 is [0, 6]
+    /// assert_eq!(inner0.into_iter().next().unwrap(), aview1(&[0, 6]));
+    /// // The first lane for axis 1 is [0, 3]
+    /// assert_eq!(inner1.into_iter().next().unwrap(), aview1(&[0, 3]));
+    /// // The first lane for axis 2 is [0, 1, 2]
+    /// assert_eq!(inner2.into_iter().next().unwrap(), aview1(&[0, 1, 2]));
+    /// ```
+    pub fn lanes(&self, axis: Axis) -> Lanes<A, D::Smaller> {
+        new_lanes(self.view(), axis)
+    }
+
+    /// Return a producer and iterable that traverses over all 1D lanes
+    /// pointing in the direction of `axis`.
+    ///
+    /// Iterator element is `ArrayViewMut1<A>` (1D read-write array view).
+    pub fn lanes_mut(&mut self, axis: Axis) -> LanesMut<A, D::Smaller>
+        where S: DataMut
+    {
+        new_lanes_mut(self.view_mut(), axis)
+    }
+
+
     /// Return an iterator that traverses over the outermost dimension
     /// and yields each subview.
     ///
@@ -598,7 +699,7 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
         iterators::new_chunk_iter_mut(self.view_mut(), axis.index(), size)
     }
 
-    /// Return a whole chunks producer (and iterable).
+    /// Return an exact chunks producer (and iterable).
     ///
     /// It produces the whole chunks of a given n-dimensional chunk size,
     /// skipping the remainder along each dimension that doesn't fit evenly.
@@ -609,13 +710,21 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
     /// **Panics** if any dimension of `chunk_size` is zero<br>
     /// (**Panics** if `D` is `IxDyn` and `chunk_size` does not match the
     /// number of array axes.)
-    pub fn whole_chunks<E>(&self, chunk_size: E) -> WholeChunks<A, D> 
+    pub fn exact_chunks<E>(&self, chunk_size: E) -> ExactChunks<A, D> 
         where E: IntoDimension<Dim=D>,
     {
-        whole_chunks_of(self.view(), chunk_size)
+        exact_chunks_of(self.view(), chunk_size)
     }
 
-    /// Return a whole chunks producer (and iterable).
+    #[doc(hidden)]
+    #[deprecated(note="Renamed to exact_chunks")]
+    pub fn whole_chunks<E>(&self, chunk_size: E) -> ExactChunks<A, D> 
+        where E: IntoDimension<Dim=D>,
+    {
+        self.exact_chunks(chunk_size)
+    }
+
+    /// Return an exact chunks producer (and iterable).
     ///
     /// It produces the whole chunks of a given n-dimensional chunk size,
     /// skipping the remainder along each dimension that doesn't fit evenly.
@@ -633,7 +742,7 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
     /// let mut a = Array::zeros((6, 7));
     ///
     /// // Fill each 2 × 2 chunk with the index of where it appeared in iteration
-    /// for (i, mut chunk) in a.whole_chunks_mut((2, 2)).into_iter().enumerate() {
+    /// for (i, mut chunk) in a.exact_chunks_mut((2, 2)).into_iter().enumerate() {
     ///     chunk.fill(i);
     /// }
     ///
@@ -647,11 +756,40 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
     ///          [6, 6, 7, 7, 8, 8, 0],
     ///          [6, 6, 7, 7, 8, 8, 0]]));
     /// ```
-    pub fn whole_chunks_mut<E>(&mut self, chunk_size: E) -> WholeChunksMut<A, D> 
+    pub fn exact_chunks_mut<E>(&mut self, chunk_size: E) -> ExactChunksMut<A, D> 
         where E: IntoDimension<Dim=D>,
               S: DataMut
     {
-        whole_chunks_mut_of(self.view_mut(), chunk_size)
+        exact_chunks_mut_of(self.view_mut(), chunk_size)
+    }
+
+    #[doc(hidden)]
+    #[deprecated(note="Renamed to exact_chunks_mut")]
+    pub fn whole_chunks_mut<E>(&mut self, chunk_size: E) -> ExactChunksMut<A, D> 
+        where E: IntoDimension<Dim=D>,
+              S: DataMut
+    {
+        self.exact_chunks_mut(chunk_size)
+    }
+
+    /// Return a window producer and iterable.
+    ///
+    /// The windows are all distinct overlapping views of size `window_size`
+    /// that fit into the array's shape.
+    /// 
+    /// Will yield over no elements if window size is larger
+    /// than the actual array size of any dimension.
+    ///
+    /// The produced element is an `ArrayView<A, D>` with exactly the dimension
+    /// `window_size`.
+    /// 
+    /// **Panics** if any dimension of `window_size` is zero.<br>
+    /// (**Panics** if `D` is `IxDyn` and `window_size` does not match the
+    /// number of array axes.)
+    pub fn windows<E>(&self, window_size: E) -> Windows<A, D>
+        where E: IntoDimension<Dim=D>
+    {
+        windows(self.view(), window_size)
     }
 
     // Return (length, stride) for diagonal
@@ -1161,8 +1299,8 @@ impl<A, S, D> ArrayBase<S, D> where S: Data<Elem=A>, D: Dimension
         // break the arrays up into their inner rows
         let n = self.ndim();
         let dim = self.raw_dim();
-        Zip::from(new_inners_mut(self.view_mut(), Axis(n - 1)))
-            .and(new_inners(rhs.broadcast_assume(dim), Axis(n - 1)))
+        Zip::from(new_lanes_mut(self.view_mut(), Axis(n - 1)))
+            .and(new_lanes(rhs.broadcast_assume(dim), Axis(n - 1)))
             .apply(move |s_row, r_row| {
                 Zip::from(s_row).and(r_row).apply(|a, b| f(a, b))
             });
