@@ -9,15 +9,18 @@
 
 #[macro_use] mod macros;
 mod chunks;
-mod inners;
+mod windows;
+mod lanes;
+pub mod iter;
 
 use std::marker::PhantomData;
 use std::ptr;
 
 use Ix1;
 
+use PrivateNew;
+
 use super::{Dimension, Ix, Ixs};
-use super::{Iter, ElementsRepr, ElementsBase, ElementsBaseMut, IterMut, IndexedIter, IndexedIterMut};
 use super::{
     ArrayBase,
     Data,
@@ -28,21 +31,27 @@ use super::{
     NdProducer,
 };
 
+pub use self::windows::{
+    Windows,
+    windows
+};
 pub use self::chunks::{
-    WholeChunks,
-    WholeChunksIter,
-    whole_chunks_of,
-    WholeChunksMut,
-    WholeChunksIterMut,
-    whole_chunks_mut_of,
+    ExactChunks,
+    ExactChunksIter,
+    exact_chunks_of,
+    ExactChunksMut,
+    ExactChunksIterMut,
+    exact_chunks_mut_of,
 };
-pub use self::inners::{
-    new_inners,
-    new_inners_mut,
-    Inners,
-    InnersMut,
+pub use self::lanes::{
+    new_lanes,
+    new_lanes_mut,
+    Lanes,
+    LanesMut,
 };
 
+use std::slice::{self, Iter as SliceIter, IterMut as SliceIterMut};
+
 /// Base for array iterators
 ///
 /// Iterator element type is `&'a A`.
@@ -254,6 +263,100 @@ clone_bounds!(
     }
 );
 
+impl<'a, A, D> PrivateNew<ArrayView<'a, A, D>> for Iter<'a, A, D>
+    where D: Dimension
+{
+    fn new(self_: ArrayView<'a, A, D>) -> Self {
+        Iter {
+            inner: if let Some(slc) = self_.into_slice() {
+                ElementsRepr::Slice(slc.iter())
+            } else {
+                ElementsRepr::Counted(self_.into_elements_base())
+            },
+        }
+    }
+}
+
+
+
+impl<'a, A, D> PrivateNew<ArrayViewMut<'a, A, D>> for IterMut<'a, A, D>
+    where D: Dimension
+{
+    fn new(self_: ArrayViewMut<'a, A, D>) -> Self {
+        IterMut {
+            inner:
+            match self_.into_slice_() {
+                Ok(x) => ElementsRepr::Slice(x.into_iter()),
+                Err(self_) => ElementsRepr::Counted(self_.into_elements_base()),
+            }
+        }
+    }
+}
+
+#[derive(Clone)]
+pub enum ElementsRepr<S, C> {
+    Slice(S),
+    Counted(C),
+}
+
+/// An iterator over the elements of an array.
+///
+/// Iterator element type is `&'a A`.
+///
+/// See [`.iter()`](../struct.ArrayBase.html#method.iter) for more information.
+pub struct Iter<'a, A: 'a, D> {
+    inner: ElementsRepr<SliceIter<'a, A>, ElementsBase<'a, A, D>>,
+}
+
+/// Counted read only iterator
+pub struct ElementsBase<'a, A: 'a, D> {
+    pub inner: Baseiter<'a, A, D>,
+}
+
+/// An iterator over the elements of an array (mutable).
+///
+/// Iterator element type is `&'a mut A`.
+///
+/// See [`.iter_mut()`](../struct.ArrayBase.html#method.iter_mut) for more information.
+pub struct IterMut<'a, A: 'a, D> {
+    inner: ElementsRepr<SliceIterMut<'a, A>, ElementsBaseMut<'a, A, D>>,
+}
+
+/// An iterator over the elements of an array.
+///
+/// Iterator element type is `&'a mut A`.
+pub struct ElementsBaseMut<'a, A: 'a, D> {
+    pub inner: Baseiter<'a, A, D>,
+}
+
+
+/// An iterator over the indexes and elements of an array.
+///
+/// See [`.indexed_iter()`](../struct.ArrayBase.html#method.indexed_iter) for more information.
+#[derive(Clone)]
+pub struct IndexedIter<'a, A: 'a, D>(ElementsBase<'a, A, D>);
+/// An iterator over the indexes and elements of an array (mutable).
+///
+/// See [`.indexed_iter_mut()`](../struct.ArrayBase.html#method.indexed_iter_mut) for more information.
+pub struct IndexedIterMut<'a, A: 'a, D>(ElementsBaseMut<'a, A, D>);
+
+impl<'a, A, D> PrivateNew<ElementsBase<'a, A, D>> for IndexedIter<'a, A, D>
+    where D: Dimension
+{
+    fn new(x: ElementsBase<'a, A, D>) -> Self {
+        IndexedIter(x)
+    }
+}
+
+impl<'a, A, D> PrivateNew<ElementsBaseMut<'a, A, D>> for IndexedIterMut<'a, A, D>
+    where D: Dimension
+{
+    fn new(x: ElementsBaseMut<'a, A, D>) -> Self {
+        IndexedIterMut(x)
+    }
+}
+
+
 impl<'a, A, D: Dimension> Iterator for Iter<'a, A, D> {
     type Item = &'a A;
     #[inline]
@@ -417,37 +520,14 @@ impl<'a, A, D> ExactSizeIterator for IndexedIterMut<'a, A, D>
 /// An iterator that traverses over all dimensions but the innermost,
 /// and yields each inner row.
 ///
-/// See [`.inner_iter()`](struct.ArrayBase.html#method.inner_iter) for more information.
-pub struct InnerIter<'a, A: 'a, D> {
+/// See [`.lanes()`](../struct.ArrayBase.html#method.lanes) for more information.
+pub struct LanesIter<'a, A: 'a, D> {
     inner_len: Ix,
     inner_stride: Ixs,
     iter: Baseiter<'a, A, D>,
 }
 
-pub fn new_inner_iter<A, D>(mut v: ArrayView<A, D>) -> InnerIter<A, D>
-    where D: Dimension
-{
-    let ndim = v.ndim();
-    if ndim == 0 {
-        InnerIter {
-            inner_len: 1,
-            inner_stride: 1,
-            iter: v.into_base_iter(),
-        }
-    } else {
-        // Set length of innerest dimension to 1, start iteration
-        let len = v.dim.last_elem();
-        let stride = v.strides.last_elem() as isize;
-        v.dim.set_last_elem(1);
-        InnerIter {
-            inner_len: len,
-            inner_stride: stride,
-            iter: v.into_base_iter(),
-        }
-    }
-}
-
-impl<'a, A, D> Iterator for InnerIter<'a, A, D>
+impl<'a, A, D> Iterator for LanesIter<'a, A, D>
     where D: Dimension
 {
     type Item = ArrayView<'a, A, Ix1>;
@@ -463,7 +543,7 @@ impl<'a, A, D> Iterator for InnerIter<'a, A, D>
     }
 }
 
-impl<'a, A, D> ExactSizeIterator for InnerIter<'a, A, D>
+impl<'a, A, D> ExactSizeIterator for LanesIter<'a, A, D>
     where D: Dimension
 {
     fn len(&self) -> usize {
@@ -471,44 +551,21 @@ impl<'a, A, D> ExactSizeIterator for InnerIter<'a, A, D>
     }
 }
 
-// NOTE: InnerIterMut is a mutable iterator and must not expose aliasing
+// NOTE: LanesIterMut is a mutable iterator and must not expose aliasing
 // pointers. Due to this we use an empty slice for the raw data (it's unused
 // anyway).
 /// An iterator that traverses over all dimensions but the innermost,
 /// and yields each inner row (mutable).
 ///
-/// See [`.inner_iter_mut()`](struct.ArrayBase.html#method.inner_iter_mut)
+/// See [`.lanes_mut()`](../struct.ArrayBase.html#method.lanes_mut)
 /// for more information.
-pub struct InnerIterMut<'a, A: 'a, D> {
+pub struct LanesIterMut<'a, A: 'a, D> {
     inner_len: Ix,
     inner_stride: Ixs,
     iter: Baseiter<'a, A, D>,
 }
 
-pub fn new_inner_iter_mut<A, D>(mut v: ArrayViewMut<A, D>) -> InnerIterMut<A, D>
-    where D: Dimension,
-{
-    let ndim = v.ndim();
-    if ndim == 0 {
-        InnerIterMut {
-            inner_len: 1,
-            inner_stride: 1,
-            iter: v.into_base_iter(),
-        }
-    } else {
-        // Set length of innerest dimension to 1, start iteration
-        let len = v.dim.last_elem();
-        let stride = v.strides.last_elem() as isize;
-        v.dim.set_last_elem(1);
-        InnerIterMut {
-            inner_len: len,
-            inner_stride: stride,
-            iter: v.into_base_iter(),
-        }
-    }
-}
-
-impl<'a, A, D> Iterator for InnerIterMut<'a, A, D>
+impl<'a, A, D> Iterator for LanesIterMut<'a, A, D>
     where D: Dimension,
 {
     type Item = ArrayViewMut<'a, A, Ix1>;
@@ -526,7 +583,7 @@ impl<'a, A, D> Iterator for InnerIterMut<'a, A, D>
     }
 }
 
-impl<'a, A, D> ExactSizeIterator for InnerIterMut<'a, A, D>
+impl<'a, A, D> ExactSizeIterator for LanesIterMut<'a, A, D>
     where D: Dimension,
 {
     fn len(&self) -> usize {
@@ -534,57 +591,6 @@ impl<'a, A, D> ExactSizeIterator for InnerIterMut<'a, A, D>
     }
 }
 
-#[cfg(next_version)]
-/// Create an InnerIter one dimension smaller than D (if possible)
-pub fn new_inner_iter_smaller<A, D>(v: ArrayView<A, D>)
-    -> InnerIter<A, D::TrySmaller>
-    where D: Dimension
-{
-    let ndim = v.ndim();
-    let len;
-    let stride;
-    let iter_v;
-    if ndim == 0 {
-        len = 1;
-        stride = 0;
-        iter_v = v.try_remove_axis(Axis(0))
-    } else {
-        len = v.dim.last_elem();
-        stride = v.strides.last_elem() as isize;
-        iter_v = v.try_remove_axis(Axis(ndim - 1))
-    }
-    InnerIter {
-        inner_len: len,
-        inner_stride: stride,
-        iter: iter_v.into_base_iter(),
-    }
-}
-
-#[cfg(next_version)]
-pub fn new_inner_iter_smaller_mut<A, D>(v: ArrayViewMut<A, D>)
-    -> InnerIterMut<A, D::TrySmaller>
-    where D: Dimension,
-{
-    let ndim = v.ndim();
-    let len;
-    let stride;
-    let iter_v;
-    if ndim == 0 {
-        len = 1;
-        stride = 0;
-        iter_v = v.try_remove_axis(Axis(0))
-    } else {
-        len = v.dim.last_elem();
-        stride = v.strides.last_elem() as isize;
-        iter_v = v.try_remove_axis(Axis(ndim - 1))
-    }
-    InnerIterMut {
-        inner_len: len,
-        inner_stride: stride,
-        iter: iter_v.into_base_iter(),
-    }
-}
-
 #[derive(Debug)]
 pub struct OuterIterCore<A, D> {
     index: Ix,
@@ -681,8 +687,8 @@ impl<A, D> DoubleEndedIterator for OuterIterCore<A, D>
 ///
 /// Iterator element type is `ArrayView<'a, A, D>`.
 ///
-/// See [`.outer_iter()`](struct.ArrayBase.html#method.outer_iter)
-/// or [`.axis_iter()`](struct.ArrayBase.html#method.axis_iter)
+/// See [`.outer_iter()`](../struct.ArrayBase.html#method.outer_iter)
+/// or [`.axis_iter()`](../struct.ArrayBase.html#method.axis_iter)
 /// for more information.
 #[derive(Debug)]
 pub struct AxisIter<'a, A: 'a, D> {
@@ -818,8 +824,8 @@ pub fn new_axis_iter<A, D>(v: ArrayView<A, D>, axis: usize)
 ///
 /// Iterator element type is `ArrayViewMut<'a, A, D>`.
 ///
-/// See [`.outer_iter_mut()`](struct.ArrayBase.html#method.outer_iter_mut)
-/// or [`.axis_iter_mut()`](struct.ArrayBase.html#method.axis_iter_mut)
+/// See [`.outer_iter_mut()`](../struct.ArrayBase.html#method.outer_iter_mut)
+/// or [`.axis_iter_mut()`](../struct.ArrayBase.html#method.axis_iter_mut)
 /// for more information.
 pub struct AxisIterMut<'a, A: 'a, D> {
     iter: OuterIterCore<A, D>,
@@ -885,12 +891,12 @@ pub fn new_axis_iter_mut<A, D>(v: ArrayViewMut<A, D>, axis: usize)
     }
 }
 
-impl<'a, A, D> NdProducer for AxisIter<'a, A, D>
-    where D: Dimension
+impl<'a, A, D: Dimension> NdProducer for AxisIter<'a, A, D>
 {
     type Item = <Self as Iterator>::Item;
     type Dim = Ix1;
-    type Elem = A;
+    type Ptr = *mut A;
+    type Stride = isize;
 
     #[doc(hidden)]
     fn layout(&self) -> ::Layout {
@@ -901,7 +907,7 @@ impl<'a, A, D> NdProducer for AxisIter<'a, A, D>
         Ix1(self.len())
     }
     #[doc(hidden)]
-    fn as_ptr(&self) -> *mut Self::Elem {
+    fn as_ptr(&self) -> Self::Ptr {
         self.iter.ptr
     }
 
@@ -910,13 +916,13 @@ impl<'a, A, D> NdProducer for AxisIter<'a, A, D>
     }
 
     #[doc(hidden)]
-    unsafe fn as_ref(&self, ptr: *mut Self::Elem) -> Self::Item {
+    unsafe fn as_ref(&self, ptr: Self::Ptr) -> Self::Item {
         ArrayView::new_(ptr,
                         self.iter.inner_dim.clone(),
                         self.iter.inner_strides.clone())
     }
     #[doc(hidden)]
-    unsafe fn uget_ptr(&self, i: &Self::Dim) -> *mut Self::Elem {
+    unsafe fn uget_ptr(&self, i: &Self::Dim) -> Self::Ptr {
         self.iter.ptr.offset(self.iter.stride * i[0] as isize)
     }
 
@@ -932,12 +938,12 @@ impl<'a, A, D> NdProducer for AxisIter<'a, A, D>
     private_impl!{}
 }
 
-impl<'a, A, D> NdProducer for AxisIterMut<'a, A, D>
-    where D: Dimension
+impl<'a, A, D: Dimension> NdProducer for AxisIterMut<'a, A, D>
 {
     type Item = <Self as Iterator>::Item;
     type Dim = Ix1;
-    type Elem = A;
+    type Ptr = *mut A;
+    type Stride = isize;
 
     #[doc(hidden)]
     fn layout(&self) -> ::Layout {
@@ -948,7 +954,7 @@ impl<'a, A, D> NdProducer for AxisIterMut<'a, A, D>
         Ix1(self.len())
     }
     #[doc(hidden)]
-    fn as_ptr(&self) -> *mut Self::Elem {
+    fn as_ptr(&self) -> Self::Ptr {
         self.iter.ptr
     }
 
@@ -957,13 +963,13 @@ impl<'a, A, D> NdProducer for AxisIterMut<'a, A, D>
     }
 
     #[doc(hidden)]
-    unsafe fn as_ref(&self, ptr: *mut Self::Elem) -> Self::Item {
+    unsafe fn as_ref(&self, ptr: Self::Ptr) -> Self::Item {
         ArrayViewMut::new_(ptr,
                            self.iter.inner_dim.clone(),
                            self.iter.inner_strides.clone())
     }
     #[doc(hidden)]
-    unsafe fn uget_ptr(&self, i: &Self::Dim) -> *mut Self::Elem {
+    unsafe fn uget_ptr(&self, i: &Self::Dim) -> Self::Ptr {
         self.iter.ptr.offset(self.iter.stride * i[0] as isize)
     }
 
@@ -988,7 +994,7 @@ impl<'a, A, D> NdProducer for AxisIterMut<'a, A, D>
 ///
 /// Iterator element type is `ArrayView<'a, A, D>`.
 ///
-/// See [`.axis_chunks_iter()`](struct.ArrayBase.html#method.axis_chunks_iter) for more information.
+/// See [`.axis_chunks_iter()`](../struct.ArrayBase.html#method.axis_chunks_iter) for more information.
 pub struct AxisChunksIter<'a, A: 'a, D> {
     iter: OuterIterCore<A, D>,
     last_ptr: *mut A,
@@ -1124,7 +1130,7 @@ macro_rules! chunk_iter_impl {
 ///
 /// Iterator element type is `ArrayViewMut<'a, A, D>`.
 ///
-/// See [`.axis_chunks_iter_mut()`](struct.ArrayBase.html#method.axis_chunks_iter_mut)
+/// See [`.axis_chunks_iter_mut()`](../struct.ArrayBase.html#method.axis_chunks_iter_mut)
 /// for more information.
 pub struct AxisChunksIterMut<'a, A: 'a, D> {
     iter: OuterIterCore<A, D>,
@@ -1153,14 +1159,14 @@ chunk_iter_impl!(AxisChunksIterMut, ArrayViewMut);
 
 send_sync_read_only!(Iter);
 send_sync_read_only!(IndexedIter);
-send_sync_read_only!(InnerIter);
+send_sync_read_only!(LanesIter);
 send_sync_read_only!(AxisIter);
 send_sync_read_only!(AxisChunksIter);
 send_sync_read_only!(ElementsBase);
 
 send_sync_read_write!(IterMut);
 send_sync_read_write!(IndexedIterMut);
-send_sync_read_write!(InnerIterMut);
+send_sync_read_write!(LanesIterMut);
 send_sync_read_write!(AxisIterMut);
 send_sync_read_write!(AxisChunksIterMut);
 send_sync_read_write!(ElementsBaseMut);
@@ -1169,18 +1175,17 @@ send_sync_read_write!(ElementsBaseMut);
 /// to deliver exactly as many items as it said it would.
 pub unsafe trait TrustedIterator { }
 
-use std::slice;
-use std::iter;
+use std;
 use linspace::Linspace;
-use indexes::Indices;
+use iter::IndicesIter;
 
 unsafe impl<F> TrustedIterator for Linspace<F> { }
 unsafe impl<'a, A, D> TrustedIterator for Iter<'a, A, D> { }
-unsafe impl<I, F> TrustedIterator for iter::Map<I, F>
+unsafe impl<I, F> TrustedIterator for std::iter::Map<I, F>
     where I: TrustedIterator { }
 unsafe impl<'a, A> TrustedIterator for slice::Iter<'a, A> { }
 unsafe impl TrustedIterator for ::std::ops::Range<usize> { }
-unsafe impl<D> TrustedIterator for Indices<D> where D: Dimension { }
+unsafe impl<D> TrustedIterator for IndicesIter<D> where D: Dimension { }
 
 
 /// Like Iterator::collect, but only for trusted length iterators
