@@ -11,7 +11,7 @@ use ndarray::{
     arr3,
 };
 use ndarray::indices;
-use itertools::free::enumerate;
+use itertools::{enumerate, zip};
 
 #[test]
 fn test_matmul_rcarray()
@@ -96,13 +96,13 @@ fn test_index()
         *elt = i;
     }
 
-    for ((i, j), a) in indices((2, 3)).zip(A.iter()) {
+    for ((i, j), a) in zip(indices((2, 3)), &A) {
         assert_eq!(*a, A[[i, j]]);
     }
 
     let vi = A.slice(&[Si(1, None, 1), Si(0, None, 2)]);
     let mut it = vi.iter();
-    for ((i, j), x) in indices((1, 2)).zip(it.by_ref()) {
+    for ((i, j), x) in zip(indices((1, 2)), &mut it) {
         assert_eq!(*x, vi[[i, j]]);
     }
     assert!(it.next().is_none());
@@ -925,7 +925,7 @@ fn test_f_order() {
     assert_eq!(c.strides(), &[3, 1]);
     assert_eq!(f.strides(), &[1, 2]);
     itertools::assert_equal(f.iter(), c.iter());
-    itertools::assert_equal(f.inner_iter(), c.inner_iter());
+    itertools::assert_equal(f.genrows(), c.genrows());
     itertools::assert_equal(f.outer_iter(), c.outer_iter());
     itertools::assert_equal(f.axis_iter(Axis(0)), c.axis_iter(Axis(0)));
     itertools::assert_equal(f.axis_iter(Axis(1)), c.axis_iter(Axis(1)));
@@ -1072,6 +1072,14 @@ fn test_default() {
     assert_eq!(b, arr0(Foo::default()));
 }
 
+#[test]
+fn test_default_ixdyn() {
+    let a = <Array<f32, IxDyn> as Default>::default();
+    let b = <Array<f32, _>>::zeros(IxDyn(&[0]));
+    assert_eq!(a, b);
+}
+
+
 #[test]
 fn test_map_axis() {
     let a = arr2(&[[1, 2, 3],
